package com.collabnet.teamforge.liquibase.util;

import java.io.InputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Properties;

import javax.xml.transform.Source;

import org.junit.Test;
import org.xmlunit.builder.DiffBuilder;
import org.xmlunit.builder.Input;
import org.xmlunit.diff.ComparisonControllers;
import org.xmlunit.diff.Diff;

import junit.framework.Assert;

public class LiquibaseXmlComparatorTest  {

	@Test
	public void testXMLEqual() throws IOException {
		Properties prop = new Properties();
		InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream("liquibase.properties");
		prop.load(inputStream);
		Source source1 = Input.fromFile(prop.getProperty("xmlSourceFile1")).build();
		Source source2 = Input.fromFile(prop.getProperty("xmlSourceFile2")).build();

		
		Diff documentDiff = DiffBuilder
	            .compare(source1)
	            .withTest(source2)
	            .checkForIdentical()
	            .ignoreComments()
	            .ignoreWhitespace()
	            .withComparisonController(ComparisonControllers.StopWhenDifferent)
	            //ignore "changeSet" element which have a child element "createSequence" 
	            .withNodeFilter(node -> !(node.getNodeName().equals(Constants.ChangeLog.Tags.CHANGE_SET) &&
	            		node.getFirstChild().getNodeName().equals(Constants.ChangeLog.Tags.CREATE_SEQUENCE)))
	            //ignore attributes "id" and "author" since liquibase generates unique id's across change sets.
	            // and there is no guarantee that the author of change sets will be the same every time.
	            .withAttributeFilter(attr -> !attr.getName().equals(Constants.ChangeLog.Attributes.ID) &&
	            		!attr.getName().equals(Constants.ChangeLog.Attributes.AUTHOR))
	            .build();
		
		String msg = Boolean.toString(documentDiff.hasDifferences());
		Files.write(Paths.get("IsDifferent.txt"), msg.getBytes());
		Assert.assertFalse(documentDiff.hasDifferences());
	}
	

}

