
package AS::Actions;

#   id => {
#      call       => # Actual sub implementing the action.
#      name       => # Name to return to AH
#      parameters => # Hashref of things wanted as input.
#   }

use Data::Dumper;

sub new {	
	my $class = shift;
	my $self  = {
		artifactStatusUpdate => {
		    name   => "Update Artifact Status", 
		    types  => ['WorkItem'],
			method => sub {
				my $AS    = shift;  # ActionServer object
				my $data  = shift;  # Input Rabbit Message
				my $args  = shift;  # Input Arguments
				my $event = $AS->json->decode($data->{'originalMessage'});
				
				main::debug(sprintf("[artifactStatusUpdate] Setting status of %s to %s",$event->{'work_item'}->{'id'}, $args->{'Status'}));
				
				my $object = $AS->TeamForge('TrackerApp','getArtifactData',
					$event->{'work_item'}->{'id'});
					
				return if ($object->{'status'} eq $args->{'Status'} ); # Avoid update if status is already right.
					
				$object->{'status'} = $args->{'Status'};
				$AS->TeamForge('TrackerApp','setArtifactData',$object,
					"Updated by rule: " . $data->{'ruleInformation'}->{'ruleName'});
			},
			parameters => {
				Status => {
					description => "Status",
					required    => 'true',
					type        => 'text',
				}
			}
			
			
		},
		updateParentArtifact => {
			name => "Update Parent Artifact Status",
		    types  => ['WorkItem'],
			method => sub {
				my $AS    = shift;  # ActionServer object
				my $data  = shift;  # Input Rabbit Message
				my $args  = shift;  # Input Arguments
				my $event = $AS->json->decode($data->{'originalMessage'});
				
				main::debug(sprintf("[updateParentArtifact] Setting parent status of %s to %s",$event->{'work_item'}->{'id'}, $args->{'Status'}));
				
				my $list = $AS->TeamForge('TrackerApp','getParentDependencyList',
					$event->{'work_item'}->{'id'});
					
			 	return unless( $list->{'dataRows'}->[0]->{'originId'});
			 	
			 	my $object = $AS->TeamForge('TrackerApp','getArtifactData',
					$list->{'dataRows'}->[0]->{'originId'});
					
				return if ($object->{'status'} eq $args->{'Status'} ); # Avoid update if status is already right.
					
				$object->{'status'} = $args->{'Status'};
				$AS->TeamForge('TrackerApp','setArtifactData',$object,
					"Updated by rule: " . $data->{'ruleInformation'}->{'ruleName'});
			},
			parameters => {
				Status => {
					description => "Status",
					required    => 'true',
					type        => 'text',
				}
			}
			
		}
		
	};
	
	bless $self;
}

sub list {
	my $self = shift;
	my $type = shift;
	my @list;
	
	for my $item (keys(%$self)) {
		push @list, $item if ( grep(/$type/, @{ $self->{ $item }->{ types }}));		
	}
	return @list;
}

sub actionName {
	my $self   = shift;
	my $action = shift;
	return $self->{ $action }->{ name };
}

sub actionParameters {
	my $self   = shift;
	my $action = shift;
	return $self->{ $action }->{ parameters };
}

sub actionMethod {
	my $self   = shift;
	my $action = shift;
	return $self->{ $action }->{ method };
}


1;